/*------------------------------------------------------------------------------
    GALE. Fine-grained parallel Genetic Algorithms for Data Mining.
    Copyright (C) 2002  Xavier Llor <xllora@illigal.ge.uiuc.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


package edu.uiuc.ge.illigal.gale.data;

import edu.uiuc.ge.illigal.gale.config.GALEConfiguration;

import java.util.Vector;
import java.util.Random;

/**
 * This class computes the instance sets for hold out.
 *
 * @author  Xavier Llor&agrave; &lt;<a href="mailto:xllora@illigal.ge.uiuc.edu">xllora@illigal.ge.uiuc.edu</a>&gt;
 * @version 0.9alpha
 * @since 0.9alpha
 */

public class HoldOutGenerator
{
   //---------------------------------------------------------------------

   /** Computes the instance sets for a stratified
	 *
	 *  @param inss the instance set to use
    *  @param fProp the hold out proportion (% training instances)
    *  @param galeCnf the GALE environment configuration object
    *  @return the train an test instance sets
    */
   public static InstanceSet [] stratifiedHoldOut ( InstanceSet inss,
	                                             	 float fProp, 
	                                                 GALEConfiguration galeCnf )
	{
		Random      rnd  = galeCnf.getRandom();
		final int  iSets = 2;
		final int   iMax = inss.getSize();
		int            i = 0;
		int         iTmp = 0;
		Instance     ins = null;
		int    iTrainSet = Math.round ( fProp*iMax );
		int     iTestSet = iMax-iTrainSet;
		String sBaseName = "GALE."+inss.getRelationName()+".holdout.seed."+galeCnf.getSeed()+".prop."+fProp;
		AttributeSet att = inss.attributeSet();

		Vector [] preisa = new Vector[2];
		preisa[0] = new Vector();
		preisa[1] = new Vector();

		int         iCls = (int)att.attribute(att.size()-1).upperBound();
		Vector [] vecArr = new Vector[iCls];
		float  [] faProp = new float[iCls];
		int    [] iaNum  = new int[iCls];

		// Vectors for Stratification
		for ( i=0 ; i<iCls ; i++ )
			vecArr[i] = new Vector();

		// Stratifying instances
		for ( i=0 ; i<iMax ; i++ ) {
			ins = inss.getInstance(i);
			vecArr[ins.getInstanceClass()].addElement(ins);
		}

		// Computing the class proportions per folder
		for ( i=0 ; i<iCls ; i++ ) {
			faProp[i] = ((float)vecArr[i].size())/iMax;
			iaNum[i]  = (int)(faProp[i]*iTrainSet);
		}

		// Filling the train set
		for ( int j=0 ; j<iCls ; j++ )
			for ( int k=0 ; k<iaNum[j] ; k++ ) {
					iTmp = rnd.nextInt(vecArr[j].size());
					preisa[0].addElement(vecArr[j].elementAt(iTmp));
					vecArr[j].remove(iTmp);
				}

		// Filling the test
		for ( i=0 ; i<iCls ; i++ ) {
			iTmp = vecArr[i].size();
			for ( int j=0 ; j<iTmp ; j++ )
				preisa[1].addElement(vecArr[i].elementAt(j));
		}

		ConstructiveInstanceSet [] isa = new ConstructiveInstanceSet[iSets];
		isa[0] = new ConstructiveInstanceSet(preisa[0].size(),sBaseName+".train",galeCnf,att);
		isa[1] = new ConstructiveInstanceSet(preisa[1].size(),sBaseName+".test",galeCnf,att);

		// TODO: randomize
		for ( int j=0 ; j<iSets ; j++ ) 
			while ( preisa[j].size()>0 ) {
				iTmp = rnd.nextInt(preisa[j].size());
				isa[j].setInstance((Instance)preisa[j].elementAt(iTmp));
				preisa[j].removeElementAt(iTmp);
			}

		return isa;
	}

   //---------------------------------------------------------------------
}
