/*------------------------------------------------------------------------------
    GALE. Fine-grained parallel Genetic Algorithms for Data Mining.
    Copyright (C) 2002  Xavier Llor <xllora@illigal.ge.uiuc.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


package edu.uiuc.ge.illigal.gale.io;

import java.util.Hashtable;
import java.io.*;

/** 
 * Implements a simple IO factory with multiple channels onto a file system.
 *
 * @author  Xavier Llor&agrave; &lt;<a href="mailto:xllora@illigal.ge.uiuc.edu">xllora@illigal.ge.uiuc.edu</a>&gt;
 * @version 0.9alpha 
 * @since   0.9alpha 
 *
 */

public class FileIOFactory
implements IOFactory
{
	//-------------------------------------------------------------------------
	
	/** The FileOutputStream array references */
	protected FileOutputStream [] fosa;

	/** The PrintStream array references */
 	protected PrintStream [] psa;
	
	/** Translation for channel ids */
	protected Hashtable htTrans;

	//-------------------------------------------------------------------------

	/**
	 * Builds an IOFactory mapped onto a file system. It allows only num
	 * channels open at a certain time,
	 *
	 * @param iMax Maximum number of concurrent channels
	 */
	public FileIOFactory ( int iMax )
	{
		fosa    = new FileOutputStream[iMax];
		psa     = new PrintStream[iMax];
		htTrans = new Hashtable();
	}

	//-------------------------------------------------------------------------

	// IOFactory interface implementation
	public int openChannel ( String sName, String sURL )
	throws IOFactoryException
	{
		int i;

		for ( i=0 ; fosa[i]!=null && i<fosa.length ; i++ );

		if ( i==fosa.length )
			throw new IOFactoryException("All channels already open");
		else {
			try {
				fosa[i] = new FileOutputStream(sURL);
				psa[i]  = new PrintStream(fosa[i]);
			}
			catch ( Exception e ) {
				throw new IOFactoryException("OCE:"+e);
			}
		}
		htTrans.put(sName,new Integer(i));
		return i;
	}

	//-------------------------------------------------------------------------
	
	// IOFactory interface implementation
	public void closeChannel ( int iDesc )
	throws IOFactoryException
	{
		try {
			if ( fosa[iDesc]!=null ) {
				psa[iDesc].close();
				fosa[iDesc].close();
	
				psa[iDesc] = null;
				fosa[iDesc] = null;
			}
		}
		catch ( Exception e ) {
			throw new IOFactoryException("CCE:"+e);
		}
	}

	//-------------------------------------------------------------------------
	
	// IOFactory interface implementation
	public void closeChannel ( String sName )
	throws IOFactoryException
	{
		Integer i = (Integer)htTrans.get(sName);
		if ( i==null ) 
			throw new IOFactoryException("CCE: the channel "+sName+" not opened");
		else
			closeChannel(i.intValue());
	}

	//-------------------------------------------------------------------------
	
	// IOFactory interface implementation
	public void closeAllChannels ()
	throws IOFactoryException
	{
		for ( int i=0, iMax=fosa.length ; i<iMax ; i++ )
			closeChannel(i);
	}

	//-------------------------------------------------------------------------

	// IOFactory interface implementation
	public void push ( int iDesc, Object obj )
	throws IOFactoryException
	{
		if ( fosa[iDesc]!=null ) 
			psa[iDesc].println(obj);
		else
			throw new IOFactoryException("Channel not available for pushing");
	}

	//-------------------------------------------------------------------------

	// IOFactory interface implementation
	public void push ( String sName, Object obj )
	throws IOFactoryException
	{
		Integer i = (Integer)htTrans.get(sName);
		if ( i==null ) 
			throw new IOFactoryException("PCE: the channel "+sName+" not opened");
		else
			push(i.intValue(),obj);
	}

	//-------------------------------------------------------------------------

}

